fileformat 8bit
set sourcestringclass String

set compile_env: 0
expectvalue /Class
doit
GsTestResult comment: 
'GsTestResult is the GemStone-specific refinement of the SUnit TestResult class.'
%
expectvalue /Class
doit
GsTestResult category: 'Kernel'
%
! ------------------- Remove existing behavior from GsTestResult
expectvalue /Metaclass3
doit
GsTestResult removeAllMethods.
GsTestResult class removeAllMethods.
%
set compile_env: 0
! ------------------- Class methods for GsTestResult
category: 'Debugging'
classmethod: GsTestResult
defectLogFile
	^ SessionTemps current
		at: #'GsTestResultDefectsLog'
		ifAbsentPut: [ 
			| f |
			f := GsFile openAppendOnServer: 'SUnitDefects.log'.
			f ifNil: [ Error signal: 'open failed' , GsFile serverErrorString ].
			f ]
%
category: 'Excluding'
classmethod: GsTestResult
excludeInSoloSession: aTestCase

"Overwritten by Ernie"

^ false
%
! ------------------- Instance methods for GsTestResult
category: 'Adding'
method: GsTestResult
addDefect: aTestCase
	"uncomment to debug test framework"
	"nil pause . "

	self logDefect: aTestCase.
	self defects add: aTestCase printString	"don't keep aTestCase alive in memory"
%
category: 'Adding'
method: GsTestResult
addError: aTestCase
  "nil pause . ""uncomment to debug test framework"
  self addDefect: aTestCase.
  super addError: aTestCase printString  "don't keep aTestCase alive in memory"
%
category: 'Adding'
method: GsTestResult
addFailure: aTestCase
  "nil pause. ""uncomment to debug test framework"
  self addDefect: aTestCase.
  super addFailure: aTestCase printString "don't keep aTestCase alive in memory"
%
category: 'Adding'
method: GsTestResult
addPass: aTestCase
  ^self passed add: aTestCase printString  "don't keep aTestCase alive in memory"
%
category: 'Accessing'
method: GsTestResult
defects
	^ defects ifNil: [ defects := NonPersistentArray new ]
%
category: 'Accessing'
method: GsTestResult
errors
	^ errors ifNil: [ errors := NonPersistentArray new ]
%
category: 'Accessing'
method: GsTestResult
failures
	^ failures ifNil: [ failures := NonPersistentArray new ]
%
category: 'Testing'
method: GsTestResult
isError: aTestCase
	"Unlike my superclass, I do not retain test instances."
	self shouldNotImplement: #isError:
%
category: 'Testing'
method: GsTestResult
isFailure: aTestCase
	"Unlike my superclass, I do not retain test instances."
	self shouldNotImplement: #isFailure:
%
category: 'Testing'
method: GsTestResult
isPassed: aTestCase
	"Unlike my superclass, I do not retain test instances."
	self shouldNotImplement: #isPassed:
%
category: 'Logging'
method: GsTestResult
logDefect: aTestCase
	(System gemEnvironmentVariable: 'resultdir')
		ifNotNil: [ :path | 
			| idx dir str ver |
			self class defectLogFile
				ifNotNil: [ :f | 
					idx := path indexOfLastByte: $/ codePoint startingAt: path size.
					dir := path copyFrom: idx + 1 to: path size.
					ver := System _gemVersion.
					idx := ver indexOf: $, startingAt: 1.
					ver := ver copyFrom: 1 to: idx - 1.
					str := ver , ', ' , System hostname , ', ' , dir , ', '
						, aTestCase printString.
					f
						nextPutAll: str lf;
						flush ] ]
		ifNil: [ 
			"assume interactive, don't log"
			 ]
%
category: 'Accessing'
method: GsTestResult
passed
	^ passed ifNil: [ passed := NonPersistentArray new ]
%
category: 'Running'
method: GsTestResult
runCase: aTestCase
" | debugBlk |
uncomment and enable to get an SUnit.log file of send...debug: of all the test cases"
" (SessionTemps current at: #Ernie_collectCases otherwise: nil) ifNotNil:[
   GsTestCase logCr: aTestCase class name , ' debug: ', aTestCase selector printString.
   ^ self
 ]."
" debugBlk := [ :ex |
   (SessionTemps current at: #Ernie_debugIgnoreErrNum otherwise: nil) ifNotNil:[:n |
     ex gsNumber == n ifFalse:[ nil pause ].
   ].
 ]. "
 (self class excludeInSoloSession: aTestCase) ifTrue:[
   GsTestCase logCr: aTestCase printString , ' -  Excluding in Solo session'.
   ^ self .
 ]. 
 [
   [  aTestCase runCase.
      self passed add: aTestCase.
   ] on: self class failure do: [:ex |
     "debugBlk value: ex . "
     self addFailure: aTestCase.
     GsTestCase logCr: 'fail - ' , ex description asString.
     self
   ].
 ] on: (Error, Notification) do: [:ex | 
   ex class defaultHandlers size > 0 ifTrue:[ ex pass ].
		"debugBlk value: ex  ."
		self addError: aTestCase.
		GsTestCase logCr: 'ERROR ', ex number asString,'  ', ex description asString .
	  GsTestCase isDebug ifFalse:[
           GsTestCase logCr:'------------------------------(
' , (GsProcess stackReportToLevel:300) , '
------------------------------)'.
   ].
   self
 ].
%
