!=========================================================================
! Copyright (C) GemTalk Systems 1986-2022.  All Rights Reserved.
!=========================================================================

set compile_env: 0
expectvalue /Class
doit
GsTestSuite comment:
'GsTestSuite is the GemStone-specific refinement of the SUnit TestSuite class.'
%

! Remove existing behavior from GsTestSuite
set class GsTestSuite
removeallmethods
removeallclassmethods

! ------------------- Class methods for GsTestSuite
! ------------------- Instance methods for GsTestSuite
category: 'other'
method: GsTestSuite
addAllTestsTo: aList

	tests do: [:each | 
		each addAllTestsTo: aList.
	].
%
category: 'other'
method: GsTestSuite
allTests

	| list |
	list := OrderedCollection new.
	self addAllTestsTo: list.
	^list.
%
category: 'other'
method: GsTestSuite
errorIfDefects

        | result list |
        result := self run.
        (list := result defects) notEmpty ifTrue: [list error: 'Found ' , list size printString , ' defects'].
        ^true.
%
category: 'other'
classmethod: GsTestSuite
firstElementOfNRS

  | firstElement dbfNRS |
      
  firstElement := SystemRepository fileNames at: 1.
  dbfNRS := (firstElement isKindOf: Array)
    ifTrue:  [ firstElement first ] "32-bit has an array of arrays of { extent, replicate}"
    ifFalse: [ firstElement].       "64-bit has an array of strings"
  ^ dbfNRS
%
category: 'other'
classmethod: GsTestSuite
logFilePath

  | path |
  path := './SUnit.log'.
  ^ path .
%
category: 'other'
method: GsTestSuite
run
  | result |
  result := GsTestResult new.
  GsTestCase openSunitLog: #filesystem .
  [self run: result] sunitEnsure: [self resources do: [:each | each reset]].
  GsTestCase flushLog .
  ^result
%
category: 'Running'
method: GsTestSuite
run: aResult

  self tests do: [:each |  "each may be either a TestSuite or a TestCase"
    self sunitChanged: each.
    each run: aResult
  ]
%
category: 'debugging'
method: GsTestSuite
_asExecString: runStr
  | str lf |
  str := String new . lf := Character lf .
  str addAll:'run'; add: lf  .
  self tests do: [:each |
    str addAll: each class name ; addAll: runStr ;
      addAll: ' #'; addAll: each selector ; addAll: ' .'; add: lf 
  ].
  str add: $% ; add: lf .
  ^ str
%

category: 'debugging'
method: GsTestSuite
asDebugString
^ self _asExecString: ' debug: '
%
category: 'debugging'
method: GsTestSuite
asRunString
^ self _asExecString: ' run: '
%

